﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Collections.ObjectModel;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.GridView;
using Telerik.Windows.Controls.DragDrop;

namespace SilverlightApplication31
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();
            testTreeListView.ItemsSource = TreeNode.Generate(5);
            this.testTreeListView.DataLoaded += new EventHandler<EventArgs>(RadTreeListView1_DataLoaded);
            this.testTreeListView.AddHandler(RadDragAndDropManager.DropInfoEvent, new EventHandler<DragDropEventArgs>(treeList_OnDropInfo));
        }

        void deleteDropTarget_OnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            e.QueryResult = true;
        }

        void favoritesDropTarget_OnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            e.QueryResult = true;
        }

        void RadTreeListView1_DataLoaded(object sender, EventArgs e)
        {
            this.testTreeListView.DataLoaded -= new EventHandler<EventArgs>(RadTreeListView1_DataLoaded);
            this.testTreeListView.ExpandAllHierarchyItems();
        }


        void treeList_OnDropInfo(object sender, DragDropEventArgs e)
        {
            if (e.Options.Status == DragStatus.DropComplete)
            {
                Collection<Object> payload = e.Options.Payload as Collection<Object>;
                TreeNode droppedItem = (TreeNode)payload[0];
                GridViewRow destinationRow = e.Options.Destination as GridViewRow;
                if (destinationRow != null)
                {
                    TreeNode targetItem = destinationRow.DataContext as TreeNode;
                    TreeListViewDropPosition relativeDropPosition = (TreeListViewDropPosition)destinationRow.GetValue(RadTreeListView.DropPositionProperty);
                    MoveItem(droppedItem, targetItem, relativeDropPosition);
                }
                else
                {
                    MoveItemToRoot(droppedItem);
                }
            }
        }

        private void MoveItemToRoot(TreeNode droppedItem)
        {
            ObservableCollection<TreeNode> parentCollection = GetParentCollection(droppedItem);
            parentCollection.Remove(droppedItem);
            droppedItem.Parent = null;
            ((ObservableCollection<TreeNode>)this.testTreeListView.ItemsSource).Add(droppedItem);

            this.testTreeListView.ExpandAllHierarchyItems();
        }

        private void MoveItem(TreeNode droppedItem, TreeNode targetItem, TreeListViewDropPosition relativeDropPosition)
        {
            if (droppedItem == targetItem)
                return;

            ObservableCollection<TreeNode> parentCollection = GetParentCollection(droppedItem);

            parentCollection.Remove(droppedItem);

            if (relativeDropPosition == TreeListViewDropPosition.Inside)
            {
                targetItem.Children.Add(droppedItem);
            }
            else if (relativeDropPosition == TreeListViewDropPosition.Before)
            {
                ObservableCollection<TreeNode> targetParentCollection = GetParentCollection(targetItem);
                targetParentCollection.Insert(targetParentCollection.IndexOf(targetItem), droppedItem);
            }
            else if (relativeDropPosition == TreeListViewDropPosition.After)
            {
                ObservableCollection<TreeNode> targetParentCollection = GetParentCollection(targetItem);
                targetParentCollection.Insert(targetParentCollection.IndexOf(targetItem) + 1, droppedItem);
            }

            this.testTreeListView.ExpandAllHierarchyItems();
        }

        private ObservableCollection<TreeNode> GetParentCollection(TreeNode droppedItem)
        {
            if (droppedItem.Parent != null)
                return droppedItem.Parent.Children;
            else
                return this.testTreeListView.ItemsSource as ObservableCollection<TreeNode>;
        }
    }

    public class TreeNode
    {
        static Random rnd = new Random();

        public string Label { get; set; }
        public ObservableCollection<TreeNode> Children { get; set; }
        public TreeNode Parent { get; set; }

        public TreeNode(bool flag)
        {
            Label = String.Format("Node{0}", rnd.Next(10000));
            Children = new ObservableCollection<TreeNode>();
            this.Children.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(Children_CollectionChanged);

            if (flag)
                for (int i = 0; i < 3; i++)
                {
                    Children.Add(new TreeNode(false) { Parent = this });
                }
        }

        public static ObservableCollection<TreeNode> Generate(int count)
        {
            ObservableCollection<TreeNode> res = new ObservableCollection<TreeNode>();
            for (int i = 0; i < count; i++)
            {
                res.Add(new TreeNode(true));
            }
            return res;
        }

        void Children_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
            {
                foreach (TreeNode node in e.NewItems)
                {
                    node.Parent = this;
                }
            }
        }
    }
}
